
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <dos.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <mem.h>

#include "cgdk.h"
#include "timer.h"
#include "csf.h"
#include "pal.h"
#include "svga.h"
#include "tm.h"
#include "keyb.h"
#include "mouse.h"
#include "car.h"
#include "file.h"
#include "lin.h"
#include "spr.h"

#define MOUSE_CURSORNAME "MOUSE.CSF"

static TCAR *carHandle;

static void help (void)
{
  printf ("Syntax is: CSFMAKE <multicsf.csf> <sprites.csf>\n"
          "                   [size_x] [size_y]\n"
          "                   [<font character table.fct>]\n"
          "                   [spacing x] [spacing y]\n"
          "\n"
          "Use size_x = size_y = 0 to autosetup sprite size (use first sprite values)\n");
}

/*
void printAsciiTab (void)
{
  int i,j;
  unsigned char c;
  char table [16*16*2];

  memset (table, ' ',sizeof table);
  for (j = 0; j < 16; j++)
  {
    for (i = 0; i < 16; i++)
    {
      c = i+j*16;
      if (c == '\n')
      {
        c = ' ';
      }
      table [2*(i+j*16)] = c;
    }
    table [2*(16+j*16)-1] = '\n';
  }
  table [0] = ' ';
  table [2*(16*16-1)] = 0;

  SVGA_putStr (0, 0, table, 255);
}
*/

#define ASCII_TAB_SPACING 24

static long asciiUsedTable [256];

static void printAsciiTab (void)
{
  int i,j;
  unsigned char c;

  for (j = 0; j < 16; j++)
  {
    for (i = 0; i < 16; i++)
    {
      c = i+j*16;
      if (c == '\n')
      {
        c = ' ';
      }
      if (asciiUsedTable [c] == 0)
      {
        SVGA_putChar (i*ASCII_TAB_SPACING, j*ASCII_TAB_SPACING, c, 255);
      }
      else
      {
        SVGA_putChar (i*ASCII_TAB_SPACING, j*ASCII_TAB_SPACING, c, 254);
      }
    }
  }
}

static char getChar (void)
{
  unsigned char c;

  do
  {
    MOUSE_updateCursor ();
    SVGA_flush ();
    c = KEYB_ascii (KEYB_get ());
    if (!c)
    {
      if (MOUSE_info.buttons.left)
      {
        while (MOUSE_info.buttons.left)
          ;
        c = (MOUSE_info.x / ASCII_TAB_SPACING) +
             16* (MOUSE_info.y / ASCII_TAB_SPACING);
      }
      if (MOUSE_info.buttons.right)
      {
        while (MOUSE_info.buttons.right)
          ;
        c = '\r';

      }
    }
    if (asciiUsedTable [c] != 0)
    {
      c = 0;
    }
  }
  while (!c);

  return c;
}

static int createFontList (const char *name, TF_list *files, short sizeX, short sizeY)
{
  int i,j,k;
  TRGBColor pal [256];
  int videoMode;
  FILE *flist;
  TCSF *mcursor;
  TCSF *csf;
  char *zoomBuf;
  unsigned char c;
  char mouseCursorName [80];
  char exitStr [256];

  strcpy (mouseCursorName, CGDK_getProgramPath (MOUSE_CURSORNAME));
  mcursor = CSF_new (carHandle, mouseCursorName);
  if (mcursor)
  {
    memset (pal, 0, sizeof pal);
    pal [255].r = 200;
    pal [255].g = 200;
    pal [255].b = 200;
    pal [254].r = 200;
    pal [254].g = 0;
    pal [254].b = 0;
    CSF_updateColors (mcursor, pal, 2, 254);

    flist = fopen (name,"wt");
    if (flist)
    {
      #ifndef __TURBOC__
        videoMode = SVGA_recommendVideoMode (800, 600, 1, VMODE_ANY);
      #else
        videoMode = VGA_320x200x256;
      #endif
      SVGA_setVideoMode (videoMode);
      PAL_clear (0, 256);
      MOUSE_setWindow (0,0,16*ASCII_TAB_SPACING,16*ASCII_TAB_SPACING);
      MOUSE_setCursor (mcursor, 0);
//      MOUSE_setSensitivity (1,1);
      for (i = 0; i < files->max; i++)
      {
        SVGA_clear (1);

        csf = CSF_new (carHandle, files->names [i]);
        if (csf)
        {
          CSF_updateColors (csf, pal, 2, 254);
          CSF_put (csf, SVGA_maxX-csf->sizeX, SVGA_maxY-csf->sizeY, 0);
          zoomBuf = SPR_4timesZoom (csf->raw, csf->sizeX, csf->sizeY);
          SVGA_putSpr (zoomBuf, SVGA_maxX-4*csf->sizeX, 0, 4*csf->sizeX, 4*csf->sizeY, 4*csf->sizeX);
          _free (zoomBuf);

          zoomBuf = SPR_2timesZoom (csf->raw, csf->sizeX, csf->sizeY);

          for (j = 0; j < 2; j++)
          {
            for (k = 0; k < 2; k++)
            {
              SVGA_putSpr (zoomBuf, SVGA_maxX-2*(2-k)*csf->sizeX, 4*csf->sizeY+j*2*csf->sizeY, 2*csf->sizeX, 2*csf->sizeY, 2*csf->sizeX);
            }
          }
          _free (zoomBuf);

          for (j = 0; j < 4; j++)
          {
            for (k = 0; k < 4; k++)
            {
              SVGA_putSpr (csf->raw, SVGA_maxX-(4-k)*csf->sizeX, 8*csf->sizeY+j*csf->sizeY, csf->sizeX, csf->sizeY, csf->sizeX);
            }
          }

          CSF_delete (csf);
        }
        else
        {
          sprintf (exitStr, "Error loading csf file %s\n", files->names [i]);
          CGDK_exit (exitStr, EXIT_FAILURE);
        }

        SVGA_putStr (0, SVGA_maxY-128,
          "Select a character or press a key corresponding for the selected sprite\n"
          "(to type upper characters use shift).\n"
          "Hold down control to select multiple characters for this sprite.\n"
          "If you do not want to select any (or further) character for this sprite,\n"
          "press enter or right mouse button.\n", 255);

        SVGA_putStr (0, SVGA_maxY-32, files->names [i], 255);
        PAL_set (pal, 0, 256);
        do
        {
          printAsciiTab ();
          MOUSE_showCursor ();
          SVGA_flush ();

          c = getChar ();
          if (c == '\r')
          {
            break;
          }
          fprintf (flist, "%c %s\n",c,files->names [i]);
          asciiUsedTable [c] = 1;

          MOUSE_hideCursor ();
          SVGA_flush ();
        }
        while (KEYB_ctrl);

        PAL_clear (0, 256);
      }
      fclose (flist);
      SVGA_done ();
      return 0;
    }
    return 1;
  }
  return 2;
}

int main (void)
{
  TF_list *files;
  int total=0;
  char *fileName;
  TCSF **sprList;
  TCSF *csf;
  char *fonListName=NULL;
  int sizeX=0;
  int sizeY=0;
  int spaceX=0;
  int spaceY=0;
  long *fonTable = NULL;
  char *fonPathTable [256];
  char s [256];
  char path [256];
  unsigned char c;
  int i;
  FILE *fonFile;

  memset (fonTable, 0, sizeof fonTable);
  memset (fonPathTable, 0, sizeof fonPathTable);
  printf ("\n"
          "CSFMAKE - single Cybertek Sprite/Font to\n"
          "          multi Cybertek Sprite/Font convertor\n"
          "\n%s\n",CGDK_copyright);

  CGDK_init ();
  TIMER_init (TIMER_OLDFREQ_DEF);

  KEYB_init ();
  carHandle = CAR_open (CAR_EMULATION);

  switch (_argc)
  {
    case 2+1:
      break;
    case 7+1:
    {
      sscanf (_argv [6], "%i", &spaceX);
      sscanf (_argv [7], "%i", &spaceY);
    }
    case 5+1:
    {
      FILE *fonFile;

      fonTable = (long *) _calloc (256,sizeof (long *),NULL);
      fonListName = strdup (_argv [5]);
    }

    case 4+1:
      sscanf (_argv [3], "%i", &sizeX);
      sscanf (_argv [4], "%i", &sizeY);
      break;

    default:
      help ();
      return 0;
  }

  files = F_listNew (_argv [2]);
  if (files->max == 0)
  {
    sprintf (exitStr, "No files matching %s found.\n",_argv [2]);
    CGDK_exit (exitStr, EXIT_FAILURE);
  }

  csf = CSF_new (carHandle, files->names [0]);
  {
    if (csf)
    {
      if (sizeX == 0)
        sizeX = csf->sizeX;
      if (sizeY == 0)
        sizeY = csf->sizeY;
    }
    else
    {
      sprintf (exitStr, "Error loading sprite %s.\n", fileName);
      CGDK_exit (exitStr, EXIT_FAILURE);
    }
    CSF_delete (csf);
  }

  if (fonListName)
  {
    if (!F_exist (fonListName))
    {
      // file list does not exist - create a new one
      if (createFontList (fonListName, files, sizeX, sizeY) != 0)
      {
        sprintf (exitStr, "Error creating font list %s.\n",fonListName);
        CGDK_exit (exitStr, EXIT_FAILURE);
      }
    }
    fonFile = fopen (fonListName, "rt");
    if (fonFile != NULL)
    {
      while (fgets (s, 255, fonFile))
      {
        sscanf (s, "%c%s", &c, &path);
        fonPathTable [c] = _fullpath (NULL, path, 255);
      }
    }
    else
    {
      sprintf (exitStr, "Error loading font list %s.\n", _argv [5]);
      CGDK_exit (exitStr, EXIT_FAILURE);
    }
  }

  sprList = (TCSF * *) _calloc (files->max+1,sizeof (long *),NULL);
  total = 0;
  while ((fileName = F_listNext (files)) != NULL)
  {
    sprList [total] = CSF_new (carHandle, fileName);
    if (sprList [total] != NULL)
    {
      total++;
      if (fonTable)
      {
        _fullpath (path, fileName, 255);
        for (i = 0; i < 256; i++)
        {
          if (strcmpi (path, fonPathTable [i]) == 0)
          {
            fonTable [i] = total;
          }
        }
      }
    }
    else
    {
      sprintf (exitStr, "Error loading sprite %s.\n", fileName);
      CGDK_exit (exitStr, EXIT_FAILURE);
    }
  }

  csf = CSF_createMulti (sprList, sizeX, sizeY, fonTable);
  if (csf)
  {
    CSF_setXSpacing (csf, spaceX);
    CSF_setYSpacing (csf, spaceY);
    if (CSF_save (csf, _argv [1])==0)
    {
      CGDK_exit ("Cybertek Sprite/Font has been successfully created.\n", EXIT_SUCCESS);
    }
    else
    {
      sprintf (exitStr, "Unable to open file %s.\n\n",_argv [1]);
      CGDK_exit (exitStr, EXIT_FAILURE);
    }
  }
  else
  {
    sprintf (exitStr, "Error creating sprite %s.\n\n",_argv [1]);
    CGDK_exit (exitStr, EXIT_FAILURE);
  }

  return 0;
}
