
#ifndef _DITH_H_
  #define _DITH_H_


  #ifdef __cplusplus
  extern "C" {
  #endif

  #include "pal.h"
  #include "car.h"

  #define DITH_MAXSTEP 127
  #define DITH_HEADERSIZE 512
  #define DITH_DESC "Cybertek Dither Table"
  #define DITH_VERSION 0x00010000
  #define DITH_ID 0x54444343
  #define DITH_DESCSIZE 64
  #define DITH_LABELSIZE 192

  typedef struct {
    char desc [DITH_DESCSIZE];
    char label [DITH_LABELSIZE];
    long id;
    long version;
    long maxStep;
    long baseColor;
    long maxColor;
    char *buf;
    char reserved [DITH_HEADERSIZE-
                   256-
                   sizeof (long)-
                   sizeof (char*)-
                   sizeof (long)-
                   sizeof (long)-
                   sizeof (long)-
                   sizeof (long)];
  } TDITH_tab ;

  extern char *DITH_shadeBuf;

  extern TDITH_tab DITH_tab;

  #define DITHER(color,shade) (((((unsigned)(unsigned char)shade))<<8)+(unsigned char)color)
  #define DITH_get(color,shade) DITH_shadeBuf [DITHER(color,shade)]
  #define DITH_set(color,shade,pcolor) DITH_shadeBuf [DITHER(color,shade)]=pcolor

  void DITH_done (void);
  int DITH_init (int maxStep); // = 0
  int DITH_save (char *fileName);
  int DITH_load (TCAR *carHandle, char *fileName);
  void DITH_fill (TRGBColor *pal, int baseColor, int maxColor,
                    int lightSmooth, int lightShift);

  extern void cdecl DITH_shadeMap (void *srcMask, int srcMaskSizeX,
                                      void *dstRaw, int dstFormatX,
                                      void *srcRaw, int srcRawSizeX, int srcRawSizeY);

  extern void cdecl DITH_shadeRaw (void *srcAndDstRaw, int srcAndDstFormatX,
                                   void *srcMask, int srcMaskFormatX,
                                   int srcMaskSizeX, int srcMaskSizeY);

  extern void cdecl DITH_shadeMask (int srcMask,
                                    void *dstRaw, int dstFormatX,
                                    void *srcRaw, int srcFormatX, int srcSizeX, int srcSizeY);

  extern void DITH_shadeScreen (char shade);

  #ifdef __cplusplus
  }
  #endif


#endif
