
#include <dos.h>
#include <mem.h>

#include "ipx.h"
#include "dpmi.h"
#include "cgdk.h"

#define IPX_INTERRUPT 0x7A
#define SPOOL_INTERRUPT 0x2F
#define IPX_INTERRUPT_CHECK 0x00

#define IPX_OPENSOCKET 0x0000
#define IPX_CLOSESOCKET 0x0001
#define IPX_GETLOCALTARGET 0x0002
#define IPX_SENDPACKET 0x0003
#define IPX_RECEIVEPACKET 0x0004
#define IPX_SCHEDULEEVENT 0x0005
#define IPX_CANCELEVENT 0x0006
#define IPX_SCHEDULESPECIALEVENT 0x0007
#define IPX_GETINTERVALMARKER 0x0008
#define IPX_GETINTERNETADDRESS 0x0009
#define IPX_DISABLECPUCONTROL 0x000A
#define IPX_CLOSECONNECTION 0x000B


#define IPX_DOSBUFLEN 1024
static char *IPX_dosBuf;

int IPX_Initialize (void)
{
  RMINFO rmi = {0};

  /* lets allocate some temporary data for IPX_dosBuf*/
  if (IPX_dosBuf == NULL)
  {
    IPX_dosBuf = (char *) DPMI_alloc (IPX_DOSBUFLEN);
    if (IPX_dosBuf == NULL)
    {
      CGDK_exit ("Not enough memory to allocate 16-bit IPX network communication buffer.\n", EXIT_FAILURE);
    }
    memset (IPX_dosBuf, 0, IPX_DOSBUFLEN);
  }

  /* check if IPX is installed */
  rmi.h.ah = IPX_INTERRUPT;
  rmi.h.al = IPX_INTERRUPT_CHECK;
  DPMI_int (SPOOL_INTERRUPT, &rmi);
  return (((signed char) rmi.h.al) == -1);
}

void IPX_done (void)
{
  if (IPX_dosBuf != NULL)
  {
    DPMI_free (IPX_dosBuf);
    IPX_dosBuf = NULL;
  }
}

int IPX_OpenSocket (int SocketNumber)
{
  RMINFO rmi = {0};

  rmi.x.ebx = IPX_OPENSOCKET;
  rmi.h.al = 0;
  rmi.x.edx = SocketNumber;

//  rmi.h.ah = rmi.h.dh;
//  rmi.h.dh = rmi.h.dl;
//  rmi.h.dl = rmi.h.ah;

  DPMI_int (IPX_INTERRUPT, &rmi);
  if (rmi.h.al)
    return -1;
  else
    return rmi.x.edx;
}

void IPX_CloseSocket (int SocketNumber)
{
  RMINFO rmi = {0};

  rmi.x.ebx = IPX_CLOSESOCKET;
  rmi.x.edx = SocketNumber;

  rmi.h.ah = rmi.h.dh;
  rmi.h.dh = rmi.h.dl;
  rmi.h.dl = rmi.h.ah;

  DPMI_int (IPX_INTERRUPT, &rmi);
}

int IPX_GetLocalTarget (TIPXAddress *addr, TNodeAddr *ImmediateAddress, short *TransportTime)
{
  RMINFO rmi = {0};

  /* copy network address to temporary DOS buffer */
  memcpy (IPX_dosBuf, addr, sizeof (TIPXAddress)-sizeof (short));

  /* buffer segment is in the allocated DOS buffer */
  rmi.x.es = DP_SEG (IPX_dosBuf);
  rmi.x.ebx = IPX_GETLOCALTARGET;
  /* network address is in the 0 offset in the DOS buffer */
  rmi.x.esi = DP_OFF (IPX_dosBuf);
  /* immediate address is on the same position as the network address */
  rmi.x.edi = DP_OFF (IPX_dosBuf);

  DPMI_int (IPX_INTERRUPT, &rmi);
  if (rmi.h.al == 0)
  {
    memcpy (ImmediateAddress, IPX_dosBuf, sizeof (TNodeAddr));
    *TransportTime = rmi.x.ecx;
    return 1;
  }
  else
  {
    memcpy (ImmediateAddress, addr->Node, sizeof (TNodeAddr));
    *TransportTime = 1;
    return 0;
  }
}

int IPX_SendPacket (TECBHeader *ECBBlock)
{
  RMINFO rmi = {0};
  int i=0;

  rmi.x.ebx = IPX_SENDPACKET;
  rmi.x.esi = DP_OFF (ECBBlock);
  rmi.x.es = DP_SEG (ECBBlock);
  ECBBlock->InUseFlag=1;

  DPMI_int (IPX_INTERRUPT, &rmi);
  while (ECBBlock->InUseFlag != 0) i++;
  abs (i);
  return (rmi.h.al == 0x00);
}

void IPX_ReceivePacket (TECBHeader *ECBBlock)
{
  RMINFO rmi = {0};

  rmi.x.ebx = IPX_RECEIVEPACKET;
  rmi.x.esi = DP_OFF (ECBBlock);
  rmi.x.es = DP_SEG (ECBBlock);

  DPMI_int (IPX_INTERRUPT, &rmi);
}

void IPX_ScheduleEvent (short DelayTime, TECBHeader *ECBBlock)
{
  RMINFO rmi = {0};

  rmi.x.ebx = IPX_SCHEDULEEVENT;
  rmi.x.eax = DelayTime;
  rmi.x.esi = DP_OFF (ECBBlock);
  rmi.x.es = DP_SEG (ECBBlock);
  DPMI_int (IPX_INTERRUPT, &rmi);
}

int IPX_CancelEvent (TECBHeader *ECBBlock)
{
  RMINFO rmi = {0};

  rmi.x.ebx = IPX_CANCELEVENT;
  rmi.x.esi = DP_OFF (ECBBlock);
  rmi.x.es = DP_SEG (ECBBlock);
  DPMI_int (IPX_INTERRUPT, &rmi);

  return rmi.h.al;
}

void IPX_ScheduleSpecialEvent (short DelayTime, TECBHeader *ECBBlock)
{
  RMINFO rmi = {0};

  rmi.x.ebx = IPX_SCHEDULESPECIALEVENT;
  rmi.x.eax = DelayTime;
  rmi.x.esi = DP_OFF (ECBBlock);
  rmi.x.es = DP_SEG (ECBBlock);
  DPMI_int (IPX_INTERRUPT, &rmi);
}


unsigned short IPX_GetIntervalMarker (void)
{
  RMINFO rmi = {0};

  rmi.x.ebx = IPX_GETINTERVALMARKER;
  DPMI_int (IPX_INTERRUPT, &rmi);
  return rmi.x.eax;
}

void IPX_GetLocalAddress (TIPXAddress *NetworkAddress)
{
  RMINFO rmi = {0};

  /* buffer segment is in the allocated DOS buffer */
  rmi.x.esi = DP_OFF (IPX_dosBuf);
  rmi.x.es = DP_SEG (IPX_dosBuf);
  rmi.x.ebx = IPX_GETINTERNETADDRESS;

  DPMI_int (IPX_INTERRUPT, &rmi);

  /* copy the to the DOS4G/W memory result */
  *NetworkAddress = * (TIPXAddress *) IPX_dosBuf;
}

void IPX_DisableCPUControl (void)
{
  RMINFO rmi = {0};

  rmi.x.ebx = IPX_DISABLECPUCONTROL;
  DPMI_int (IPX_INTERRUPT, &rmi);
}

void IPX_CloseConnection (TIPXAddress *NetworkAddress)
{
  RMINFO rmi = {0};

  // copy network address to temporary DOS buffer
  * (TIPXAddress *) IPX_dosBuf = * (TIPXAddress *) NetworkAddress;

  rmi.x.ebx = IPX_CLOSECONNECTION;
  rmi.x.esi = DP_OFF (IPX_dosBuf);

  /* buffer segment is in the allocated DOS buffer */
  rmi.x.es = DP_SEG  (IPX_dosBuf);

  DPMI_int (IPX_INTERRUPT, &rmi);
}

/*

BYTE SPXInitialize (BYTE *MajorRevisionNumber,
                    BYTE *MinorRevisionNumber,
                    WORD *MaxConnections,
                    WORD *AvailableConnections)

{
  *MajorRevisionNumber = 0;
  *MinorRevisionNumber = 0;
  *MaxConnections = 10;
  *AvailableConnections = 9;
  _SI = 0;
  _DI = 0;
  _AL = 0;
  _BX = 0x10;
  geninterrupt (0x7A);
  return (!_AL);
}

int SPXEstablishConnection (BYTE RetryCount, BYTE WatchDog,
                            WORD *ConnectionID, ECB *ECBBlock)
{
  _DI = 0;
  _ES = FP_SEG (ECBBlock);
  _SI = FP_OFF (ECBBlock);
  _AL = RetryCount;
  _AH = WatchDog;
  _BX = 0x11;
  asm   push bp;
  geninterrupt (0x7A);
  asm   pop bp;
  *ConnectionID = _DX;
  return (_AL);
}

void SPXListenForPacket (BYTE RetryCount, BYTE WatchDog, ECB *ECBBlock)
{
  _DI = 0;
  _ES = FP_SEG (ECBBlock);
  _SI = FP_OFF (ECBBlock);
  _AL = RetryCount;
  _AH = WatchDog;
  _BX = 0x12;
  geninterrupt (0x7A);
  return;
}

void SPXTerminateConnection (WORD ConnectionID, ECB *ECBBlock)
{
  _DI = 0;
  _ES = FP_SEG (ECBBlock);
  _SI = FP_OFF (ECBBlock);
  _DX = ConnectionID;
  _BX = 0x13;
  geninterrupt (0x7A);
  return;
}

void SPXAbortConnection (WORD ConnectionID)
{
  _SI = 0;
  _DI = 0;
  _DX = ConnectionID;
  _BX = 0x14;
  geninterrupt (0x7A);
  return;
}

void SPXSendSequencedPacket (WORD ConnectionID, ECB *ECBBlock)
{
  _DI = 0;
  _ES = FP_SEG (ECBBlock);
  _SI = FP_OFF (ECBBlock);
  _DX = ConnectionID;
  _BX = 0x16;
  geninterrupt (0x7A);
  return;
}

void SPXListenForSequencedPacket (ECB *ECBBlock)
{
  _DI = 0;
  _ES = FP_SEG (ECBBlock);
  _SI = FP_OFF (ECBBlock);
  _BX = 0x17;
  geninterrupt (0x7A);
  return;
}

/* -------------------------------------------- */
/* -------- Procedury programu Shell ---------- */
/* -------------------------------------------- */

/*
int GetInternetAddress (WORD ConnectionNumber, char *NetworkNumber,
                        char *PhysicalNodeAddress, WORD *SocketNumber)
{
  BYTE RetCode;
  struct
  {
    WORD Request;
    BYTE Network [4];
    BYTE Node [6];
    WORD Socket;
  } ReceivePacket;

  struct {
    WORD Request;
    BYTE R2;
    WORD ConNum;
  } SendPacket;

  SendPacket.R2 = 19;
  SendPacket.ConNum = ConnectionNumber;
  SendPacket.Request = 2;

  ReceivePacket.Request = 12;

  asm    push ds;
  _DS = FP_SEG (&SendPacket);
  _SI = FP_OFF (&SendPacket);
  _ES = FP_SEG (&ReceivePacket);
  _DI = FP_OFF (&ReceivePacket);
  _AH = 227;
  geninterrupt (0x21);
  asm    pop ds
  RetCode = _AL;
  memcpy (NetworkNumber,ReceivePacket.Network,4);
  memcpy (PhysicalNodeAddress,ReceivePacket.Node,6);
  *SocketNumber = ReceivePacket.Socket;
  return (RetCode);
}

int GetConnectionNumber (void)
{
_AH = 0xDC;
geninterrupt (0x21);
return (_AL);
}

*/
