
#include <mem.h>

#include "svga.h"
#include "mir.h"
#include "lin.h"
#include "dith.h"
#include "vsb.h"
#include "lin.h"

#define UNDEFINED -22222

#include "mirl_.cpp"
#include "mirc_.cpp"

static char *MIR_shadeBuf;
//static char *MIR_pool;

void MIR_done (void)
{
  if (MIR_shadeBuf)
  {
    _free (MIR_shadeBuf);
    MIR_shadeBuf = NULL;
  }
  MIRL_done ();
//  if (MIR_pool)
//  {
//    _free (MIR_pool);
//    MIR_pool = NULL;
//  }
}

void MIR_init (int maxLayer, int maxSize)
{
  MIR_done ();
  MIR_shadeBuf = (char *) _alloc (maxSize);
  if (MIR_shadeBuf == NULL)
  {
    CGDK_exit ("Not enough memory for mirror shade buffer.\n", EXIT_FAILURE);
  }
  MIRL_init (maxLayer);
//  MIR_pool = (char *) _alloc (4*sizeX*sizeY);
}

void MIR_changeSpr (TMIR *mir, void *newSpr)
{
  mir->spr = (char *) newSpr;
  if ((mir->type & MIR_TYPE_SHADEBOTH) == MIR_TYPE_SHADENONE)
  {
    mir->raw = mir->spr;
  }
  mir->status |= MIR_STATUS_REDRAW;
}


TMIR *MIR_new (void *raw, int sizeX, int sizeY, int layer, int type, char *shade)
{
  TMIR *mir;
  int size;

  if ((mir = new TMIR) != 0)
  {
    mir->x = mir->y = mir->curX = mir->curY = 0;
    mir->layer = -1;
    mir->sizeX = sizeX;
    mir->sizeY = sizeY;
    mir->size = sizeX*sizeY;
    mir->spr = (char *) raw;
    if (type & MIR_TYPE_STATIC)
    {
      size = (mir->sizeX+SVGA_maxXPlus)*(mir->sizeY+SVGA_maxYPlus);
      if (size >= 4*mir->size)
      {
        size = 4*mir->size;
        type -= MIR_TYPE_STATIC;
      }
    }
    else
    {
      size = 4*mir->size;
    }

    mir->pool = (char *) _alloc (size);
    if (mir->pool == NULL)
    {
      delete mir;
      return NULL;
    }
    mir->shade = shade;
    mir->type = type;
    MIRL_insert (mir, layer);
    switch (type & MIR_TYPE_SHADEBOTH)
    {
      case MIR_TYPE_SHADENONE:
      {
        mir->raw = mir->spr;
        break;
      }
      case MIR_TYPE_SHADEMAP:
      case MIR_TYPE_SHADEBOTH:
      case MIR_TYPE_SHADEOWN:
      {
        mir->raw = (char *) _alloc (mir->size);
        if (mir->raw == NULL)
        {
          MIRL_delete (mir);
          _free (mir->pool);
          delete mir;
          return NULL;
        }
        break;
      }
    }
    mir->status = MIR_STATUS_RESTORE;
    return mir;
  }
  return NULL;
}


void MIR_delete (TMIR *mir)
{
  if (mir)
  {
    mir->status |= MIR_STATUS_DELETE;
    MIR_restore (mir);
  }
}

void MIR_free (TMIR *mir)
{
  if (mir->raw != mir->spr)
  {
    if (mir->raw)
      _free (mir->raw);
  }
  if (mir->pool)
  {
    _free (mir->pool);
  }
  delete mir;
}

static void MIR_shadeSlow (TMIR *mir, int mirsizeX, int mirsizeY, int x, int y)
{
  switch (mir->type & MIR_TYPE_SHADEBOTH)
  {
    case MIR_TYPE_SHADEMAP:
    {
      VSB_sget (MIR_shadeBuf, x, y, mirsizeX, mirsizeY, mirsizeX);
      DITH_shadeMap (MIR_shadeBuf, mirsizeX, mir->raw, mirsizeX, mir->spr, mirsizeX, mirsizeY);
      break;
    }

    case MIR_TYPE_SHADEOWN:
    {
      memset (MIR_shadeBuf, 0, mir->size);
      MIRL_get (mir, MIR_shadeBuf, mir->x, mir->y, mirsizeX,mirsizeY);
      DITH_shadeMap (MIR_shadeBuf, mirsizeX, mir->raw, mirsizeX, mir->shade, mirsizeX, mirsizeY);
      LIN_movSpr_ (mir->raw, mirsizeX, mir->spr, mirsizeX, mirsizeX, mirsizeY);
      break;
    }
    case MIR_TYPE_SHADEBOTH:
    {
      memset (MIR_shadeBuf, 0, mir->size);
      MIRL_get (mir, MIR_shadeBuf, mir->x, mir->y, mirsizeX,mirsizeY);
        // ziska se kompletni pozadi MIRu
      DITH_shadeMap (mir->shade, mirsizeX, mir->raw, mirsizeX, MIR_shadeBuf, mirsizeX, mirsizeY);
        // vystinuje se a ulozi do mir->raw
      VSB_sget (MIR_shadeBuf, x, y, mirsizeX, mirsizeY, mirsizeX);
        // ziska se stin pozadi na kterem lezi
      DITH_shadeMap (MIR_shadeBuf, mirsizeX, MIR_shadeBuf, mirsizeX, mir->spr, mirsizeX, mirsizeY);
        // mir->spr se tim vystinuje a ulozi do MIR_shadeBuf
      LIN_movSpr_ (mir->raw, mirsizeX, MIR_shadeBuf, mirsizeX, mirsizeX, mirsizeY);
        // vystinovany sprite se prepise pres mir->raw
      break;
    }
    case MIR_TYPE_SHADENONE:
    {
      break;
    }
  }
}

void MIR_shade (TMIR *mir)
{
  MIR_shadeSlow (mir, mir->sizeX, mir->sizeY, mir->x, mir->y);
}


static void MIR_putNew (TMIR *mir,int x, int y, int sizeX, int sizeY)
{
  char *pool = mir->pool;

  MIR_shadeSlow (mir, sizeX, sizeY, x, y);
  memcpy (pool, mir->raw, mir->size);
  MIRL_putSpr (mir, pool, x, y, sizeX, sizeY);
}


static void MIR_putOld (TMIR *mir, int mirx, int miry, int sizeX, int sizeY)
{
  char *pool;

  if (mir->type & MIR_TYPE_STATIC)
  {
    pool = mir->pool;
  }
  else
  {
    pool = mir->pool + mir->size;
  }

  MIRL_get (mir, pool, mirx, miry, sizeX, sizeY);
  MIRL_putWin (mir, pool, mirx, miry, sizeX, sizeY);
}


static void MIR_updateSlow (TMIR *mir, int mirx, int miry, int sizeX, int sizeY, int x, int y)
{
  MIR_putOld (mir,mirx,miry,sizeX,sizeY);
  MIR_putNew (mir,x,y,sizeX,sizeY);
}

void MIR_restore (TMIR *mir)
{
//  if (mir != NULL)
//  {
//    if ((mir->status & MIR_STATUS_RESTORE) == 0)
//    {
  MIR_update (mir, UNDEFINED, UNDEFINED);

//    }
//  }
}


static void MIR_shadeFast (TMIR *mir, char *mirraw, int mirsizeX, int mirsizeY,
                                  int mirxdx, int mirydy,
                                  char *pool,int poolSizeX)
{
  switch (mir->type & MIR_TYPE_SHADEBOTH)
  {
    case MIR_TYPE_SHADEOWN:
    {
      DITH_shadeRaw (mir->shade, mirsizeX, pool, poolSizeX, mirsizeX, mirsizeY);
      break;
    }
    case MIR_TYPE_SHADEBOTH:
    {
      DITH_shadeRaw (mir->shade, mirsizeX, pool, poolSizeX, mirsizeX, mirsizeY);
    }
    case MIR_TYPE_SHADEMAP:
    {
      VSB_sget (MIR_shadeBuf, mirxdx, mirydy, mirsizeX, mirsizeY, mirsizeX);
      DITH_shadeMap (MIR_shadeBuf, mirsizeX, mirraw, mirsizeX, mir->spr, mirsizeX, mirsizeY);
      break;
    }
  }
}


static void MIR_updateFast (TMIR *mir, int x, int y, int sizeX, int sizeY,
                           int dx, int dy)
{
///  int x = mir->x;
//  int y = mir->y;
//  int sizeX = mir->sizeX;
//  int sizeY = mir->sizeY;
  char *raw = mir->raw;
  char *pool = mir->pool;
  int poolSizeX, poolSizeY;
  int xdx;
  int ydy;

  if (dx >= 0)
  {
    poolSizeX = sizeX+dx;

    if (dy >= 0)
    {
      // right down
      poolSizeY = sizeY+dy;

      MIRL_get (mir, pool, x, y, poolSizeX, poolSizeY);

      MIR_shadeFast (mir, raw, sizeX, sizeY, x+dx, y+dy, pool+dx+dy*poolSizeX, poolSizeX);


      LIN_movSpr_ (pool+dx+dy*poolSizeX, poolSizeX,
                   raw, sizeX, sizeX, sizeY);

      MIRL_putWin (mir, pool, x, y, poolSizeX, poolSizeY);
    }
    else
    {
      // right up
      poolSizeY = sizeY-dy;
      ydy = y+dy;

      MIRL_get (mir, pool, x, ydy, poolSizeX, poolSizeY);

      MIR_shadeFast (mir, raw, sizeX, sizeY, x+dx, y-dy, pool+dx, poolSizeX);

      LIN_movSpr_ (pool+dx, poolSizeX, mir->raw, sizeX, sizeX, sizeY);

      MIRL_putWin (mir, pool, x, ydy, poolSizeX, poolSizeY);
    }
  }
  else
  {
    poolSizeX = sizeX-dx;
    xdx = x+dx;

    // left down
    if (dy >= 0)
    {
      poolSizeY = sizeY+dy;


      MIRL_get (mir, pool, xdx, y, poolSizeX, poolSizeY);

      MIR_shadeFast (mir, raw, sizeX, sizeY, x-dx, y+dy, pool+dy*poolSizeX, poolSizeX);

      LIN_movSpr_ (pool+dy*poolSizeX, poolSizeX,
                   raw, sizeX, sizeX, sizeY);

      MIRL_putWin (mir, pool, xdx, y, poolSizeX, poolSizeY);
    }
    else
    {
      poolSizeY = sizeY-dy;
      ydy = y+dy;

      MIRL_get (mir, pool, xdx, ydy, poolSizeX, poolSizeY);

      MIR_shadeFast (mir, raw, sizeX, sizeY, x-dx, y-dy, pool, poolSizeX);

      LIN_movSpr_ (pool, poolSizeX, mir->raw, sizeX, sizeX, sizeY);

      MIRL_putWin (mir, pool, xdx, ydy, poolSizeX, poolSizeY);
    }
  }
}


void MIR_updateCurPos (TMIR *mir)
{
  int curx, cury;

  if (SVGA_curX == 0 && SVGA_dx == 0 && SVGA_curXPrev == 0)
    curx = 0;
  else
    curx = mir->curX-SVGA_curX;  // screen shift x

  if (SVGA_curY == 0 && SVGA_dy == 0 && SVGA_curYPrev == 0)
    cury = 0;
  else
    cury = mir->curY-SVGA_curY;  // screen shift y

  SVGA_virtual2RealDelta (&curx, &cury);

  mir->x += curx;
  mir->y += cury;

  mir->curX = SVGA_curX;
  mir->curY = SVGA_curY;
}


void MIRC_update (TMIR *mir, int x, int y)
{
  int curx, cury;
  int dx, dy;
  int mirx, miry;
  int mirsizeX, mirsizeY;
  int mirxdx, mirydy;

  if ((mir != NULL) && (mir->spr != NULL))
  {
    if (x == UNDEFINED)
    {
      if ((mir->status & MIR_STATUS_RESTORE) == 0)
      {
        MIR_putOld (mir, mir->x, mir->y, mir->sizeX, mir->sizeY);
        mir->status |= MIR_STATUS_RESTORE;
      }

      if (mir->status & MIR_STATUS_DELETE)
      {
        MIRL_delete (mir);
        MIR_free (mir);
      }
    }
    else
    {
      mirx = mir->x;
      mirsizeX = mir->sizeX;
      if ((mirx+mirsizeX > -SVGA_virtualXD2) && (mirx-SVGA_virtualX < +SVGA_virtualXD2))
      {
        miry = mir->y;
        mirsizeY = mir->sizeY;
        if ((miry+mirsizeY > -SVGA_virtualYD2) && (miry-SVGA_virtualY < +SVGA_virtualYD2))
        {
          if (!(mir->status & MIR_STATUS_RESTORE))
          {
            dx = x - mirx;      // sprite shift x
            dy = y - miry;      // sprite shift y

            if (
                 (dx || dy) ||
                 (mir->type & MIR_TYPE_SHADEOWN) ||
                 (mir->status & MIR_STATUS_REDRAW) ||
                 ( (SVGA_dx < 0) && ((mirx+SVGA_dx) <= 0) && ((mirx+mirsizeX+SVGA_dx) >= 0) ) ||
                 ( (SVGA_dy < 0) && ((miry+SVGA_dy) <= 0) && ((miry+mirsizeY+SVGA_dy) >= 0) ) ||
                 ( (SVGA_dx > 0) && ((mirx+SVGA_dx) <= SVGA_maxX) && ((mirx+mirsizeX+SVGA_dx) >= SVGA_maxX) ) ||
                 ( (SVGA_dy > 0) && ((miry+SVGA_dy) <= SVGA_maxY) && ((miry+mirsizeY+SVGA_dy) >= SVGA_maxY) )
               )
            {
              if ( (abs (dx) < mirsizeX) && (abs (dy) < mirsizeY)
    //   && (0)
                 )
              {
                MIR_updateFast (mir, mirx, miry, mirsizeX, mirsizeY, dx, dy);
              }
              else
              {
                MIR_updateSlow (mir, mirx, miry, mirsizeX, mirsizeY, x, y);
              }
            }
          }
          else
          {
            mir->curX = SVGA_curX;
            mir->curY = SVGA_curY;
            MIR_putNew (mir, x, y, mirsizeX, mirsizeY);
            mir->status &= -1 ^ MIR_STATUS_RESTORE;
          }
        }
      }
      mir->x = x;
      mir->y = y;
    }
    mir->status &= -1 ^ (MIR_STATUS_REDRAW | MIR_STATUS_UPDATE);
  }
}


//void MIR_updateDelta (int dx, int dy)
//{
//  MIR_update (mir->x+dx+SVGA_dx, mir->y+dy+SVGA_dy);
//}
//
